rpms/shadow-utils/devel shadow-4.0.14-goodname.patch, NONE, 1.1 shadow-4.0.14-redhat.patch, NONE, 1.1 .cvsignore, 1.9, 1.10 shadow-utils.spec, 1.64, 1.65 sources, 1.9, 1.10 shadow-4.0.13-audit-update.patch, 1.2, NONE shadow-4.0.13-auditUserdel.patch, 1.1, NONE shadow-4.0.13-goodname.patch, 1.1, NONE shadow-4.0.13-redhat.patch, 1.2, NONE

fedora-cvs-commits at redhat.com fedora-cvs-commits at redhat.com
Fri Jan 6 09:56:59 UTC 2006


Author: pvrabec

Update of /cvs/dist/rpms/shadow-utils/devel
In directory cvs.devel.redhat.com:/tmp/cvs-serv28471

Modified Files:
	.cvsignore shadow-utils.spec sources 
Added Files:
	shadow-4.0.14-goodname.patch shadow-4.0.14-redhat.patch 
Removed Files:
	shadow-4.0.13-audit-update.patch 
	shadow-4.0.13-auditUserdel.patch shadow-4.0.13-goodname.patch 
	shadow-4.0.13-redhat.patch 
Log Message:
upgrade 4.0.14


shadow-4.0.14-goodname.patch:
 libmisc/chkname.c |   39 ++++++++++++++++++++++++---------------
 man/useradd.8     |    2 --
 2 files changed, 24 insertions(+), 17 deletions(-)

--- NEW FILE shadow-4.0.14-goodname.patch ---
--- shadow-4.0.14/man/useradd.8.goodname	2006-01-05 17:54:26.000000000 +0100
+++ shadow-4.0.14/man/useradd.8	2006-01-05 18:01:18.000000000 +0100
@@ -166,8 +166,6 @@
 Similarly, if the username already exists in an external user database such as NIS,
 \fBuseradd\fR
 will deny the user account creation request.
-.PP
-Usernames must begin with a lower case letter or an underscore, and only lower case letters, underscores, dashes, and dollar signs may follow. In regular expression terms: [a\-z_][a\-z0\-9_\-]*[$]
 .SH "FILES"
 .TP
 \fI/etc/passwd\fR
--- shadow-4.0.14/libmisc/chkname.c.goodname	2005-08-31 19:24:57.000000000 +0200
+++ shadow-4.0.14/libmisc/chkname.c	2006-01-05 17:59:45.000000000 +0100
@@ -18,16 +18,24 @@
 static int good_name (const char *name)
 {
 	/*
-	 * User/group names must match [a-z_][a-z0-9_-]*[$]
-	 */
-	if (!*name || !((*name >= 'a' && *name <= 'z') || *name == '_'))
+         * User/group names must match gnu e-regex:
+         *    [a-zA-Z0-9_.][a-zA-Z0-9_.-]{0,30}[a-zA-Z0-9_.$-]?
+         *
+         * as a non-POSIX, extension, allow "$" as the last char for
+         * sake of Samba 3.x "add machine script"
+         */
+	if (!*name || !((*name >= 'a' && *name <= 'z')
+             || (*name >= 'A' && *name <= 'Z')
+             || (*name >= '0' && *name <= '9')
+             || *name == '_' || *name == '.'))
 		return 0;
 
 	while (*++name) {
-		if (!((*name >= 'a' && *name <= 'z') ||
-		      (*name >= '0' && *name <= '9') ||
-		      *name == '_' || *name == '-' ||
-		      (*name == '$' && *(name + 1) == '\0')))
+		if (!(  (*name >= 'a' && *name <= 'z')
+		     || (*name >= 'A' && *name <= 'Z')
+                     || (*name >= '0' && *name <= '9')
+                     || *name == '_' || *name == '.' || *name == '-'
+                     || (*name == '$' && *(name + 1) == '\0')))
 			return 0;
 	}
 
@@ -43,10 +51,9 @@
 #endif
 
 	/*
-	 * User names are limited by whatever utmp can
-	 * handle (usually max 8 characters).
+	 * User names are limited by whatever utmp can handle.
 	 */
-	if (strlen (name) > sizeof (ut.ut_user))
+	if (strlen(name) + 1 > sizeof(ut.ut_user))
 		return 0;
 
 	return good_name (name);
@@ -54,11 +61,13 @@
 
 int check_group_name (const char *name)
 {
-	/*
-	 * Arbitrary limit for group names - max 16
-	 * characters (same as on HP-UX 10).
-	 */
-	if (strlen (name) > 16)
+#if HAVE_UTMPX_H
+	struct utmpx ut;
+#else
+	struct utmp ut;
+#endif
+
+	if (strlen(name) + 1 > sizeof(ut.ut_user))
 		return 0;
 
 	return good_name (name);

shadow-4.0.14-redhat.patch:
 man/groupadd.8 |    7 +++-
 man/useradd.8  |   22 +++++++++++--
 src/groupadd.c |   19 +++++++++--
 src/useradd.c  |   94 ++++++++++++++++++++++++++++++++++++++-------------------
 4 files changed, 103 insertions(+), 39 deletions(-)

--- NEW FILE shadow-4.0.14-redhat.patch ---
--- shadow-4.0.14/man/groupadd.8.redhat	2006-01-03 17:44:56.000000000 +0100
+++ shadow-4.0.14/man/groupadd.8	2006-01-04 15:12:43.000000000 +0100
@@ -11,7 +11,7 @@
 groupadd \- Create a new group
 .SH "SYNOPSIS"
 .HP 9
-\fBgroupadd\fR [\-g\ \fIGID\fR\ [\-o]] [\-f] [\-K\ \fIKEY\fR=\fIVALUE\fR] \fIgroup\fR
+\fBgroupadd\fR [\-g\ \fIgid\fR\ [\-o]] [\-r] [\-f] [\-K\ \fIKEY\fR=\fIVALUE\fR] \fIgroup\fR
 .SH "DESCRIPTION"
 .PP
 The
@@ -29,10 +29,13 @@
 \fB\-g\fR
 is turned off).
 .TP
+\fB-r\fR
+This flag instructs \fBgroupadd\fR to add a system account.  The first available \fIgid\fR lower than 499 will be automatically selected unless the \fB-g\fR option is also given on the command line. This is an option added by Red Hat.
+.TP
 \fB\-g\fR \fIGID\fR
 The numerical value of the group's ID. This value must be unique, unless the
 \fB\-o\fR
-option is used. The value must be non\-negative. The default is to use the smallest ID value greater than 999 and greater than every other group. Values between 0 and 999 are typically reserved for system accounts.
+option is used. The value must be non\-negative. The default is to use the smallest ID value greater than 500 and greater than every other group. Values between 0 and 499 are typically reserved for system accounts.
 .TP
 \fB\-h\fR, \fB\-\-help\fR
 Display help message and exit.
--- shadow-4.0.14/man/useradd.8.redhat	2006-01-03 17:44:57.000000000 +0100
+++ shadow-4.0.14/man/useradd.8	2006-01-04 15:01:33.000000000 +0100
@@ -22,7 +22,7 @@
 \fB\-D\fR
 option, the
 \fBuseradd\fR
-command creates a new user account using the values specified on the command line and the default values from the system. Depending on command line options, the useradd command will update system files and may also create the new user's home directory and copy initial files.
+command creates a new user account using the values specified on the command line and the default values from the system. Depending on command line options, the useradd command will update system files and may also create the new user's home directory and copy initial files. The version provided with Red Hat Linux will create a group for each user added to the system by default.
 .SH "OPTIONS"
 .PP
 The options which apply to the
@@ -58,8 +58,7 @@
 The number of days after a password expires until the account is permanently disabled. A value of 0 disables the account as soon as the password has expired, and a value of \-1 disables the feature. The default value is \-1.
 .TP
 \fB\-g\fR, \fB\-\-gid\fR \fIGROUP\fR
-The group name or number of the user's initial login group. The group name must exist. A group number must refer to an already existing group. The default group number is 1 or whatever is specified in
-\fI/etc/default/useradd\fR.
+The group name or number of the user's initial login group. The group name must exist. A group number must refer to an already existing group.
 .TP
 \fB\-G\fR, \fB\-\-groups\fR \fIGROUP1\fR[\fI,GROUP2,...\fR[\fI,GROUPN\fR]]]
 A list of supplementary groups which the user is also a member of. Each group is separated from the next by a comma, with no intervening whitespace. The groups are subject to the same restrictions as the group given with the
@@ -68,6 +67,9 @@
 .TP
 \fB\-h\fR, \fB\-\-help\fR
 Display help message and exit.
+.TP 
+\fB-M\fR
+The user's home directory will not be created, even if the system wide settings from \fI/etc/login.defs\fR is to create home dirs.
 .TP
 \fB\-m\fR, \fB\-\-create\-home\fR
 The user's home directory will be created if it does not exist. The files contained in
@@ -85,6 +87,9 @@
 option is only valid in conjunction with the
 \fB\-m\fR
 option. The default is to not create the directory and to not copy any files.
+.TP 
+\fB-n\fR
+A group having the same name as the user being added to the system will be created by default. This option will turn off this Red Hat Linux specific behavior. When this option is used, users by default will be placed in whatever group is specified in \fI/etc/default/useradd\fR. If no default group is defined, group 1 will be used.
 .TP
 \fB\-K\fR, \fB\-\-key\fR \fIKEY\fR=\fIVALUE\fR
 Overrides /etc/login.defs defaults (UID_MIN, UID_MAX, UMASK, PASS_MAX_DAYS and others).
@@ -106,6 +111,9 @@
 \fB\-p\fR, \fB\-\-password\fR \fIPASSWORD\fR
 The encrypted password, as returned by
 \fBcrypt\fR(3). The default is to disable the account.
+.TP 
+\fB-r\fR
+This flag is used to create a system account. That is, a user with a UID lower than the value of UID_MIN defined in \fI/etc/login.defs\fR and whose password does not expire. Note that \fBuseradd\fR will not create a home directory for such an user, regardless of the default setting in \fI/etc/login.defs\fR. You have to specify \fB-m\fR option if you want a home directory for a system account to be created. This is an option added by Red Hat
 .TP
 \fB\-s\fR, \fB\-\-shell\fR \fISHELL\fR
 The name of the user's login shell. The default is to leave this field blank, which causes the system to select the default login shell.
@@ -149,6 +157,8 @@
 The system administrator is responsible for placing the default user files in the
 \fI/etc/skel/\fR
 directory.
+.br
+This version of useradd was modified by Red Hat to suit Red Hat user/group conventions.
 .SH "CAVEATS"
 .PP
 You may not add a user to a NIS group. This must be performed on the NIS server.
@@ -169,9 +179,15 @@
 \fI/etc/group\fR
 group account information
 .TP
+\fI/etc/gshadow\fR
+secure group information
+.TP
 \fI/etc/default/useradd\fR
 default information
 .TP
+\fI/etc/login.defs\fR
+system-wide settings
+.TP
 \fI/etc/skel/\fR
 directory containing default files
 .TP
--- shadow-4.0.14/src/groupadd.c.redhat	2005-12-06 21:24:03.000000000 +0100
+++ shadow-4.0.14/src/groupadd.c	2006-01-04 15:01:45.000000000 +0100
@@ -74,6 +74,7 @@
 static int oflg = 0;		/* permit non-unique group ID to be specified with -g */
 static int gflg = 0;		/* ID value for the new group */
 static int fflg = 0;		/* if group already exists, do nothing and exit(0) */
+static int rflg = 0;            /* for adding system accounts (Red Hat) */
 
 /* local function prototypes */
 static void usage (void);
@@ -100,6 +101,7 @@
 			   "Options:\n"
 			   "  -f, --force		force exit with success status if the specified\n"
 			   "				group already exists\n"
+			   "  -r,			create system account\n"
 			   "  -g, --gid GID		use GID for the new group\n"
 			   "  -h, --help			display this help message and exit\n"
 			   "  -K, --key KEY=VALUE		overrides /etc/login.defs defaults\n"
@@ -197,8 +199,13 @@
 	const struct group *grp;
 	gid_t gid_min, gid_max;
 
-	gid_min = getdef_unum ("GID_MIN", 1000);
-	gid_max = getdef_unum ("GID_MAX", 60000);
+	if (!rflg) {
+		gid_min = getdef_unum ("GID_MIN", 500);
+                gid_max = getdef_unum ("GID_MAX", 60000);
+        } else {
+                gid_min = 1;
+                gid_max = getdef_unum ("GID_MIN", 500) - 1;
+        }
 
 	/*
 	 * Start with some GID value if the user didn't provide us with
@@ -412,7 +419,7 @@
 		};
 
 		while ((c =
-			getopt_long (argc, argv, "fg:hK:o", long_options,
+			getopt_long (argc, argv, "frg:hK:o", long_options,
 				     &option_index)) != -1) {
 			switch (c) {
 			case 'f':
@@ -425,6 +432,12 @@
 				 */
 				fflg++;
 				break;
+	                case 'r':
+	                        /*
+	                         * create a system group
+	                         */
+	                        rflg++;
+	                        break;
 			case 'g':
 				gflg++;
 				if (!isdigit (optarg[0]))
Index: .cvsignore
===================================================================
RCS file: /cvs/dist/rpms/shadow-utils/devel/.cvsignore,v
retrieving revision 1.9
retrieving revision 1.10
diff -u -r1.9 -r1.10
--- .cvsignore	22 Oct 2005 12:38:17 -0000	1.9
+++ .cvsignore	6 Jan 2006 09:56:57 -0000	1.10
@@ -1 +1 @@
-shadow-4.0.13.tar.bz2
+shadow-4.0.14.tar.bz2


Index: shadow-utils.spec
===================================================================
RCS file: /cvs/dist/rpms/shadow-utils/devel/shadow-utils.spec,v
retrieving revision 1.64
retrieving revision 1.65
diff -u -r1.64 -r1.65
--- shadow-utils.spec	9 Dec 2005 22:43:13 -0000	1.64
+++ shadow-utils.spec	6 Jan 2006 09:56:57 -0000	1.65
@@ -2,25 +2,21 @@
 %define WITH_SELINUX 1
 %endif
 
-%define utf8_man_pages 1
-
 Summary: Utilities for managing accounts and shadow password files.
 Name: shadow-utils
-Version: 4.0.13
-Release: 4.1
+Version: 4.0.14
+Release: 1
 Epoch: 2
 URL: http://shadow.pld.org.pl/
 Source0: ftp://ftp.pld.org.pl/software/shadow/shadow-%{version}.tar.bz2
 Source1: shadow-970616.login.defs
 Source2: shadow-970616.useradd
-Patch0: shadow-4.0.13-redhat.patch
+Patch0: shadow-4.0.14-redhat.patch
 Patch1: shadow-4.0.3-noinst.patch
 Patch2: shadow-4.0.11.1-vipw.patch
-Patch3: shadow-4.0.13-goodname.patch
+Patch3: shadow-4.0.14-goodname.patch
 Patch4: shadow-4.0.13-newgrpPwd.patch
 Patch5: shadow-4.0.12-lOption.patch
-Patch6: shadow-4.0.13-audit-update.patch 
-Patch7: shadow-4.0.13-auditUserdel.patch
 License: BSD
 Group: System Environment/Base
 BuildRequires: autoconf, automake, libtool, gettext-devel
@@ -51,46 +47,10 @@
 %patch3 -p1 -b .goodname
 %patch4 -p1 -b .newgrpPwd
 %patch5 -p1 -b .lOption
-%patch6 -p1 -b .audit
-%patch7 -p1 -b .auditUserdel
 
 rm po/*.gmo
 rm po/stamp-po
 
-# Recode man pages from euc-jp to UTF-8.
-manconv() {
-flags="$-"
-set +x
-incode=$1
-outcode=$2
-shift 2
-for page in $* ; do
-	if ! iconv -f ${outcode} -t ${outcode} ${page} > /dev/null 2> /dev/null ; then
-		if iconv -f ${incode} -t ${outcode} ${page} > /dev/null 2> /dev/null ; then
-			iconv -f ${incode} -t ${outcode} ${page} > ${page}.tmp && \
-			cat ${page}.tmp > ${page} && \
-			rm ${page}.tmp
-		fi
-	fi
-done
-set -"$flags"
-}
-%if %{utf8_man_pages}
-manconv euc-jp utf-8 man/ja/*.*
-manconv iso-8859-1 utf-8 man/de/*
-manconv iso-8859-1 utf-8 man/fr/*
-manconv iso-8859-1 utf-8 man/it/*
-manconv iso-8859-1 utf-8 man/pt_BR/*
-manconv iso-8859-2 utf-8 man/hu/*
-manconv iso-8859-2 utf-8 man/pl/*
-manconv iso-8859-2 utf-8 man/cs/*
-manconv iso-8859-1 utf-8 man/es/*
-manconv koi8-u  utf-8 man/ru/*
-manconv euc-kr utf-8 man/ko/*.*
-manconv gb2312 utf-8 man/zh_CN/*.*
-manconv big5 utf-8 man/zh_TW/*.*
-%endif
-
 libtoolize --force
 aclocal
 automake -a
@@ -135,7 +95,7 @@
 rm $RPM_BUILD_ROOT/%{_sbindir}/logoutd
 rm $RPM_BUILD_ROOT/%{_sbindir}/vipw
 rm $RPM_BUILD_ROOT/%{_sbindir}/vigr
-
+rm $RPM_BUILD_ROOT/%{_sbindir}/nologin
 rm $RPM_BUILD_ROOT/%{_mandir}/*/man1/id.*
 rm $RPM_BUILD_ROOT/%{_mandir}/man1/chfn.*
 rm $RPM_BUILD_ROOT/%{_mandir}/*/man1/chfn.*
@@ -167,6 +127,8 @@
 rm $RPM_BUILD_ROOT/%{_mandir}/*/man8/vipw.*
 rm $RPM_BUILD_ROOT/%{_mandir}/man8/vigr.*
 rm $RPM_BUILD_ROOT/%{_mandir}/*/man8/vigr.*
+rm $RPM_BUILD_ROOT/%{_mandir}/*/man8/sulogin.*
+rm $RPM_BUILD_ROOT/%{_mandir}/*/man3/pw_auth.*
 
 %find_lang shadow
 
@@ -175,7 +137,7 @@
 
 %files -f shadow.lang
 %defattr(-,root,root)
-%doc NEWS doc/HOWTO doc/LICENSE README doc/README.linux
+%doc NEWS doc/HOWTO README
 %dir /etc/default
 %attr(0644,root,root)	%config /etc/login.defs
 %attr(0600,root,root)	%config /etc/default/useradd
@@ -210,6 +172,7 @@
 %{_mandir}/man5/login.defs.5*
 %{_mandir}/*/man5/login.defs.5*
 %{_mandir}/man5/gshadow.5*
+%{_mandir}/*/man5/gshadow.5*
 %{_mandir}/man5/faillog.5*
 %{_mandir}/*/man5/faillog.5*
 %{_mandir}/man8/adduser.8*
@@ -234,6 +197,9 @@
 %{_mandir}/*/man8/faillog.8*
 
 %changelog
+* Fri Jan 06 2006 Peter Vrabec <pvrabec at redhat.com> 2:4.0.14-1
+- upgrade
+
 * Fri Dec 09 2005 Jesse Keating <jkeating at redhat.com>
 - rebuilt
 


Index: sources
===================================================================
RCS file: /cvs/dist/rpms/shadow-utils/devel/sources,v
retrieving revision 1.9
retrieving revision 1.10
diff -u -r1.9 -r1.10
--- sources	22 Oct 2005 12:38:17 -0000	1.9
+++ sources	6 Jan 2006 09:56:57 -0000	1.10
@@ -1 +1 @@
-5c6a105d89afb0900922260e9c5650cc  shadow-4.0.13.tar.bz2
+903f55cf05bbe082617d3337743792fb  shadow-4.0.14.tar.bz2


--- shadow-4.0.13-audit-update.patch DELETED ---


--- shadow-4.0.13-auditUserdel.patch DELETED ---


--- shadow-4.0.13-goodname.patch DELETED ---


--- shadow-4.0.13-redhat.patch DELETED ---




More information about the fedora-cvs-commits mailing list
--- shadow-4.0.14/src/useradd.c.redhat	2005-12-15 16:06:28.000000000 +0100
+++ shadow-4.0.14/src/useradd.c	2006-01-04 15:01:33.000000000 +0100
@@ -81,7 +81,7 @@
 static gid_t def_group = 100;
 static const char *def_gname = "other";
 static const char *def_home = "/home";
-static const char *def_shell = "";
+static const char *def_shell = "/sbin/nologin";
 static const char *def_template = SKEL_DIR;
 static const char *def_create_mail_spool = "no";
 
@@ -93,7 +93,7 @@
 #define	VALID(s)	(strcspn (s, ":\n") == strlen (s))
 
 static const char *user_name = "";
-static const char *user_pass = "!";
+static const char *user_pass = "!!";
 static uid_t user_id;
 static gid_t user_gid;
 static const char *user_comment = "";
@@ -124,8 +124,10 @@
     Gflg = 0,			/* secondary group set for new account */
     kflg = 0,			/* specify a directory to fill new user directory */
     mflg = 0,			/* create user's home directory if it doesn't exist */
-    nflg = 0,			/* create a group having the same name as the user */
+    Mflg = 0,                   /* do NOT create user's home directory no matter what */
+    nflg = 0,                   /* do NOT create a group having the same name as the user */
     oflg = 0,			/* permit non-unique user ID to be specified with -u */
+    rflg = 0,                   /* create a system account */
     sflg = 0,			/* shell program for new account */
     uflg = 0;			/* specify user ID for new account */
 
@@ -628,6 +630,8 @@
 			   "  -K, --key KEY=VALUE		overrides /etc/login.defs defaults\n"
 			   "  -m, --create-home		create home directory for the new user\n"
 			   "				account\n"
+			   "  -M,                       do not create user's home directory(overrides /etc/login.defs)\n"
+			   "  -r,                       create system account\n"
 			   "  -o, --non-unique		allow create user with duplicate\n"
 			   "				(non-unique) UID\n"
 			   "  -p, --password PASSWORD	use encrypted password for the new user\n"
@@ -679,11 +683,20 @@
 	spent->sp_namp = (char *) user_name;
 	spent->sp_pwdp = (char *) user_pass;
 	spent->sp_lstchg = time ((time_t *) 0) / SCALE;
-	spent->sp_min = scale_age (getdef_num ("PASS_MIN_DAYS", -1));
-	spent->sp_max = scale_age (getdef_num ("PASS_MAX_DAYS", -1));
-	spent->sp_warn = scale_age (getdef_num ("PASS_WARN_AGE", -1));
-	spent->sp_inact = scale_age (def_inactive);
-	spent->sp_expire = scale_age (user_expire);
+	if (!rflg) {
+		spent->sp_min = scale_age (getdef_num ("PASS_MIN_DAYS", -1));
+		spent->sp_max = scale_age (getdef_num ("PASS_MAX_DAYS", -1));
+		spent->sp_warn = scale_age (getdef_num ("PASS_WARN_AGE", -1));
+		spent->sp_inact = scale_age (def_inactive);
+		spent->sp_expire = scale_age (user_expire);
+	}
+	else {
+		spent->sp_min = scale_age(-1);
+                spent->sp_max = scale_age(-1);
+                spent->sp_warn = scale_age(-1);
+                spent->sp_inact = scale_age(-1);
+        	spent->sp_expire = scale_age(-1);
+	}
 	spent->sp_flag = -1;
 }
 
@@ -831,8 +844,14 @@
 	const struct passwd *pwd;
 	uid_t uid_min, uid_max;
 
-	uid_min = getdef_unum ("UID_MIN", 1000);
-	uid_max = getdef_unum ("UID_MAX", 60000);
+        if (!rflg) {
+                uid_min = getdef_unum ("UID_MIN", 500);
+                uid_max = getdef_unum ("UID_MAX", 60000);
+        }
+        else {
+                uid_min = 1;
+                uid_max = getdef_unum ("UID_MIN", 500) - 1;
+        }
 
 	/*
 	 * Start with some UID value if the user didn't provide us with
@@ -862,7 +881,7 @@
 #endif
 			exit (E_NAME_IN_USE);
 		}
-		if (uflg && user_id == pwd->pw_uid) {
+		if (!oflg && uflg && user_id == pwd->pw_uid) {
 			fprintf (stderr, _("%s: UID %u is not unique\n"),
 				 Prog, (unsigned int) user_id);
 #ifdef WITH_AUDIT
@@ -1016,7 +1115,7 @@
 			{NULL, 0, NULL, '\0'}
 		};
 		while ((c =
-			getopt_long (argc, argv, "b:c:d:De:f:g:G:k:K:mMop:s:u:",
+			getopt_long (argc, argv, "b:c:d:De:f:g:G:k:K:mMnrop:s:u:",
 				     long_options, NULL)) != -1) {
 			switch (c) {
 			case 'b':
@@ -1188,6 +1287,15 @@
 				user_id = get_uid (optarg);
 				uflg++;
 				break;
+                        case 'n':
+                                nflg++;
+                                break;
+                        case 'r':
+                                rflg++;
+                                break;
+                        case 'M':
+                                Mflg++;
+                                break;
 			default:
 				usage ();
 			}
@@ -1195,6 +1303,9 @@
 		}
 	}
 
+        if (mflg && Mflg) /* the admin is not decided .. create or not ? */
+               usage();
+
 	/*
 	 * Certain options are only valid in combination with others.
 	 * Check it here so that they can be specified in any order.
@@ -1700,6 +1811,14 @@
 	}
 #endif				/* USE_PAM */
 
+        if (!rflg) /* for system accounts defaults are ignored and we
+                    * do not create a home dir -- gafton */
+        	if (getdef_bool("CREATE_HOME"))
+               		mflg = 1;
+
+        if (Mflg) /* absolutely sure that we do not create home dirs */
+        	mflg = 0;
+
 	/*
 	 * See if we are messing with the defaults file, or creating
 	 * a new user.
@@ -1730,7 +1849,7 @@
 	 * to that group, use useradd -g username username.
 	 * --bero
 	 */
-	if (!gflg) {
+	if ( !(nflg || gflg) ) {
 		if (getgrnam (user_name)) {
 			fprintf (stderr,
 				 _
@@ -1779,25 +1910,18 @@
 				 ("%s: warning: the home directory already exists.\n"
 				  "Not copying any file from skel directory into it.\n"),
 				 Prog);
-
-	} else if (getdef_str ("CREATE_HOME")) {
-		/*
-		 * RedHat added the CREATE_HOME option in login.defs in their
-		 * version of shadow-utils (which makes -m the default, with
-		 * new -M option to turn it off). Unfortunately, this
-		 * changes the way useradd works (it can be run by scripts
-		 * expecting some standard behaviour), compared to other
-		 * Unices and other Linux distributions, and also adds a lot
-		 * of confusion :-(.
-		 * So we now recognize CREATE_HOME and give a warning here
-		 * (better than "configuration error ... notify administrator"
-		 * errors in every program that reads /etc/login.defs). -MM
-		 */
-		fprintf (stderr,
-			 _
-			 ("%s: warning: CREATE_HOME not supported, please use -m instead.\n"),
-			 Prog);
 	}
+        /* Warning removed to protect the innocent. */
+        /*
+         * The whole idea about breaking some stupid scripts by creating a new
+         * variable is crap - I could care less about the scripts. Historically
+         * adduser type programs have always created the home directories and
+         * I don't like the idea of providing a script when we can fix the
+         * binary itself. And if the scripts are using the right options to the
+         * useradd then they will not break. If not, they depend on unspecified
+         * behavior and they will break, but they were broken anyway to begin
+         * with --gafton
+         */
 
 	create_mail ();
 

